/*
 * Copyright (c) 2004-2007 Erez Zadok
 * Copyright (c) 2004-2007 Stony Brook University
 * Copyright (c) 2004-2007 The Research Foundation of State University of New York
 * Copyright (c) 2004-2007 Nikolai Joukov
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package, or get one from ftp://ftp.filesystems.org/pub/fist/COPYING.
 *
 * This Copyright notice must be kept intact and distributed with all
 * fistgen sources INCLUDING sources generated by fistgen.
 */
/*
 *  $Id: fist.h,v 1.10 2007/02/02 06:32:41 ezk Exp $
 */

#ifndef __FIST_H_
#define __FIST_H_

/*
 * KERNEL ONLY CODE:
 */
#ifdef __KERNEL__
#include <linux/config.h>
#include <linux/version.h>
#include "debug.h"

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
#ifdef CONFIG_MODVERSIONS
#define MODVERSIONS
#include <linux/modversions.h>
#endif /* CONFIG_MODVERSIONS */
#include <linux/locks.h>
#else
#include <asm/statfs.h>
#endif //#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/string.h>
#include <linux/stat.h>
#include <linux/errno.h>
#include <linux/smp_lock.h>
#include <linux/file.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <linux/poll.h>
#include <linux/list.h>
#include <linux/init.h>
#include <linux/pagemap.h>
#include <linux/swap.h>

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#include <linux/mount.h>
#include <linux/namei.h>
#endif

#include <asm/system.h>
#include <asm/segment.h>
#include <asm/mman.h>
#include <linux/pagemap.h>

/*
 * MACROS:
 */

/* macros to manage changes between kernels */
# define INODE_DATA(i)	(&(i)->i_data)

#define MIN(x,y) ((x < y) ? (x) : (y))
#define MAXPATHLEN PATH_MAX

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,5)
#define lookup_one_len(a,b,c) lookup_one(a,b)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,8)
#define generic_file_llseek default_llseek
#endif

#ifndef SEEK_SET
#define SEEK_SET 0
#endif /* not SEEK_SET */

#ifndef SEEK_CUR
#define SEEK_CUR 1
#endif /* not SEEK_CUR */

#ifndef SEEK_END
#define SEEK_END 2
#endif /* not SEEK_END */



/*
 * TYPEDEFS:
 */
typedef struct dentry dentry_t;
typedef struct file file_t;
typedef struct inode inode_t;
typedef inode_t vnode_t;
typedef struct page page_t;
typedef struct qstr qstr_t;
typedef struct super_block super_block_t;
typedef super_block_t vfs_t;
typedef struct vm_area_struct vm_area_t;

/*
 * EXTERNALS:
 */

#define FPPF(str,page) printk("PPF %s 0x%x/%d: Lck:%d Err:%d Ref:%d Upd:%d Other::%d:%d:%d:%d:\n", \
		str, \
		(int) page, \
		(int) page->index, \
		(PageLocked(page) ? 1 : 0), \
		(PageError(page) ? 1 : 0), \
		(PageReferenced(page) ? 1 : 0), \
		(Page_Uptodate(page) ? 1 : 0), \
		(PageDecrAfter(page) ? 1 : 0), \
		(PageSlab(page) ? 1 : 0), \
		(PageSwapCache(page) ? 1 : 0), \
		(PageReserved(page) ? 1 : 0) \
		)
#define EZKDBG printk("EZK %s:%d:%s\n",__FILE__,__LINE__,__FUNCTION__)
#if 0
# define EZKDBG1 printk("EZK %s:%d\n",__FILE__,__LINE__)
#else
# define EZKDBG1
#endif

//#define FIST_MALLOC_DEBUG 1

#ifdef FIST_MALLOC_DEBUG
extern void *replayfs_kmalloc(size_t len, int flag, int line, const char *file);
extern void replayfs_kfree(void *ptr, int line, const char *file);
extern void *replayfs_vmalloc(size_t size,int line,const char* file);
extern void replayfs_vfree(void* ptr,int line,const char* file);
#define KMALLOC(size,flag) replayfs_kmalloc((size),(flag),__LINE__,__FILE__)
#define KFREE(ptr) replayfs_kfree((ptr),__LINE__,__FILE__)
#define VMALLOC(size) replayfs_vmalloc((size),__LINE__,__FILE__)
#define VFREE(ptr) replayfs_vfree((ptr),__LINE__,__FILE__);
#else /* not FIST_MALLOC_DEBUG */
# define KMALLOC(a,b)	kmalloc((a),(b))
# define KFREE(a)	kfree((a))
#define VMALLOC(size) vmalloc((size))
#define VFREE(ptr) vfree((ptr))
#endif /* not FIST_MALLOC_DEBUG */

extern int fist_get_debug_value(void);
extern int fist_set_debug_value(int val);
extern void fist_dprint_internal(int level, char *str,...);
extern void fist_print_dentry(char *str, const dentry_t *dentry);
extern void fist_print_inode(char *str, const inode_t *inode);
extern void fist_print_file(char *str, const file_t *file);
extern void fist_print_buffer_flags(char *str, struct buffer_head *buffer);
extern void fist_print_page_flags(char *str, page_t *page);
extern void fist_print_page_bytes(char *str, page_t *page);
extern void fist_print_pte_flags(char *str, const page_t *page);
extern void fist_checkinode(inode_t *inode, char *msg);
extern void fist_print_sb(char *str, const super_block_t *sb);
extern char *add_indent(void);
extern char *del_indent(void);

#define STATIC
#define ASSERT(EX)	\
do {	\
    if (!(EX)) {	\
	printk(KERN_CRIT "ASSERTION FAILED: %s at %s:%d (%s)\n", #EX,	\
	       __FILE__, __LINE__, __FUNCTION__);	\
	(*((char *)0))=0;	\
    }	\
} while (0)
/* same ASSERT, but tell me who was the caller of the function */
#define ASSERT2(EX)	\
do {	\
    if (!(EX)) {	\
	printk(KERN_CRIT "ASSERTION FAILED (caller): %s at %s:%d (%s)\n", #EX,	\
	       file, line, func);	\
	(*((char *)0))=0;	\
    }	\
} while (0)

#define dprintk(format, args...) printk(KERN_DEBUG format, ##args)
#define fist_dprint(level, str, args...) fist_dprint_internal(level, KERN_DEBUG str, ## args)
#if DEBUG_VALUES
#define print_entry_location() fist_dprint(4, "%sIN:  %s %s:%d\n", add_indent(), __FUNCTION__, __FILE__, __LINE__)
#define print_exit_location() fist_dprint(4, "%s OUT: %s %s:%d\n", del_indent(), __FUNCTION__, __FILE__, __LINE__)
#define print_exit_status(status) fist_dprint(4, "%s OUT: %s %s:%d, STATUS: %d\n", del_indent(), __FUNCTION__, __FILE__, __LINE__, status)
#define print_exit_pointer(status) \
do { \
  if (IS_ERR(status)) \
    fist_dprint(4, "%s OUT: %s %s:%d, RESULT: %ld\n", del_indent(), __FUNCTION__, __FILE__, __LINE__, PTR_ERR(status)); \
  else \
    fist_dprint(4, "%s OUT: %s %s:%d, RESULT: 0x%x\n", del_indent(), __FUNCTION__, __FILE__, __LINE__, PTR_ERR(status)); \
} while (0)
#else
#define print_entry_location()
#define print_exit_location()
#define print_exit_status(status)
#define print_exit_pointer(status)
#endif
#define sbt(sb) ((sb)->s_type->name)

#endif /* __KERNEL__ */


#define MAX(x,y) ((x > y) ? (x) : (y))
/*
 * DEFINITIONS FOR USER AND KERNEL CODE:
 * (Note: ioctl numbers 1--9 are reserved for fistgen, the rest
 *  are auto-generated automatically based on the user's .fist file.)
 */
# define FIST_IOCTL_GET_DEBUG_VALUE	_IOR(0x15, 1, int)
# define FIST_IOCTL_SET_DEBUG_VALUE	_IOW(0x15, 2, int)

#endif /* not __FIST_H_ */

/*
 * Local variables:
 * c-basic-offset: 4
 * End:
 */
