/*
 * Copyright (c) 2003-2007 Erez Zadok
 * Copyright (c) 2003-2007 Stony Brook University
 * Copyright (c) 2003-2007 The Research Foundation of State University of New York
 * Copyright (c) 2004-2007 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2007 Gopala Surya
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package, or get one from ftp://ftp.filesystems.org/pub/fist/COPYING.
 *
 * This Copyright notice must be kept intact and distributed with all
 * fistgen sources INCLUDING sources generated by fistgen.
 */
/*
 * tracer.h:
 * This file declares all functions that
 * are external to FiST and used within
 * FiST generated tracefs
 */

#ifndef __TRACER_H
#define __TRACER_H

#include "common.h"

int evaluate(struct trace_destination *dest, op_t op, struct inode *inode, struct dentry *dentry);

/* int add_destination(struct trace_sb *sb, struct trace_data *td); */

int tracefs_ioctl_add_odrivers(inode_t *inode, file_t *file, unsigned int cmd, unsigned long arg);
int tracefs_ioctl_add_adrivers(inode_t *inode, file_t *file, unsigned int cmd, unsigned long arg);
int tracefs_ioctl_add_rule(inode_t *inode, file_t *file, unsigned int cmd, unsigned long arg);
int tracefs_ioctl_del_odrivers(inode_t *inode, file_t *file, unsigned int cmd, unsigned long arg);
int tracefs_ioctl_del_adrivers(inode_t *inode, file_t *file, unsigned int cmd, unsigned long arg);
int tracefs_ioctl_del_rule(inode_t *inode, file_t *file, unsigned int cmd, unsigned long arg);
int tracefs_ioctl_teardown(inode_t *inode, file_t *file, unsigned int cmd, unsigned long arg);
int tracefs_ioctl_start_trace(inode_t *inode, file_t *file, unsigned int cmd, unsigned long arg);
int tracefs_ioctl_stop_trace(inode_t *inode, file_t *file, unsigned int cmd, unsigned long arg);

/* These are the hooks from the file system to the tracing subsystem */
/* inode ops */
void trace_open_pre(struct trace_sb *sb, struct inode *inode, struct file *file, void **context);
void trace_open_post(struct trace_sb *sb, struct inode *inode, struct file *file, int ret, void **context);
void trace_create_pre(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int mode, void **context);
void trace_create_post(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int mode, int ret, void **context);
void trace_lookup_pre(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, void **context);
void trace_lookup_post(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, struct dentry *new_dentry, void **context);
void trace_link_pre(struct trace_sb *sb, struct dentry *old_dentry, struct inode *dir, struct dentry *new_dentry, void **context);
void trace_link_post(struct trace_sb *sb, struct dentry *old_dentry, struct inode *dir, struct dentry *new_dentry, int ret, void **context);
void trace_unlink_pre(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, void **context);
void trace_unlink_post(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int ret, void **context);
void trace_symlink_pre(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, const char *symname, void **context);
void trace_symlink_post(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, const char *symname, int ret, void **context);
void trace_mkdir_pre(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int mode, void **context);
void trace_mkdir_post(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int mode, int ret, void **context);
void trace_rmdir_pre(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, void **context);
void trace_rmdir_post(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int ret, void **context);
void trace_mknod_pre(struct trace_sb *sb, struct inode *dir, struct dentry *denry, int mode, int dev, void **context);
void trace_mknod_post(struct trace_sb *sb, struct inode *dir, struct dentry *denry, int mode, int dev, int ret, void **context);
void trace_rename_pre(struct trace_sb *sb, struct inode *old_dir, struct dentry *old_dentry, struct inode *new_dir, struct dentry *new_dentry, void **context);
void trace_rename_post(struct trace_sb *sb, struct inode *old_dir, struct dentry *old_dentry, struct inode *new_dir, struct dentry *new_dentry, int ret, void **context);
void trace_readlink_pre(struct trace_sb *sb, struct dentry *dentry, char *buf, int bufsiz, void **context);
void trace_readlink_post(struct trace_sb *sb, struct dentry *dentry, char *buf, int bufsiz, int ret, void **context);
void trace_follow_link_pre(struct trace_sb *sb, struct dentry *dentry, struct nameidata *nd, void **context);
void trace_follow_link_post(struct trace_sb *sb, struct dentry *dentry, struct nameidata *nd, int ret, void **context);
void trace_permission_pre(struct trace_sb *sb, struct inode *inode, int mask, void **context);
void trace_permission_post(struct trace_sb *sb, struct inode *inode, int mask, int ret, void **context);
void trace_inode_revalidate_pre(struct trace_sb *sb, struct dentry *dentry, void **context);
void trace_inode_revalidate_post(struct trace_sb *sb, struct dentry *dentry, int ret, void **context);
void trace_setattr_pre(struct trace_sb *sb, struct dentry *dentry, struct iattr *ia, void **context);
void trace_setattr_post(struct trace_sb *sb, struct dentry *dentry, struct iattr *ia, int ret, void **context);

/* file ops */
void trace_llseek_pre(struct trace_sb *sb, struct file *file, loff_t offset, int origin, void **context);
void trace_llseek_post(struct trace_sb *sb, struct file *file, loff_t offset, int origin, loff_t ret, void **context);
void trace_read_pre(struct trace_sb *sb, file_t *file, char *buf, size_t count, loff_t *ppos, void **context);
void trace_read_post(struct trace_sb *sb, file_t *file, char *buf, size_t count, loff_t *ppos, ssize_t ret, void **context);
void trace_write_pre(struct trace_sb *sb, file_t *file, const char *buf, size_t count, loff_t *ppos, void **context);
void trace_write_post(struct trace_sb *sb, file_t *file, const char *buf, size_t count, loff_t *ppos, ssize_t ret, void **context);
void trace_readdir_pre(struct trace_sb *sb, file_t *file, void *dirent, filldir_t filldir, int count, void **context);
void trace_readdir_post(struct trace_sb *sb, file_t *file, void *firent, filldir_t filldir, int count, int ret, void **context);
void trace_poll_pre(struct trace_sb *sb, file_t *file, poll_table *wait, void **context);
void trace_poll_post(struct trace_sb *sb, file_t *file, poll_table *wait, int ret, void **context);
void trace_ioctl_pre(struct trace_sb *sb, inode_t *inode, file_t *file, unsigned int cmd, unsigned long arg, void **context);
void trace_ioctl_post(struct trace_sb *sb, inode_t *inode, file_t *file, unsigned int cmd, unsigned long arg, int ret, void **context);
void trace_mmap_pre(struct trace_sb *sb, file_t *file, vm_area_t *vma, void **context);
void trace_mmap_post(struct trace_sb *sb, file_t *file, vm_area_t *vma, int ret, void **context);
void trace_open_pre(struct trace_sb *sb, inode_t *inode, file_t *file, void **context);
void trace_open_post(struct trace_sb *sb, inode_t *inode, file_t *file, int ret, void **context);
void trace_flush_pre(struct trace_sb *sb, file_t *file, void **context);
void trace_flush_post(struct trace_sb *sb, file_t *file, int ret, void **context);
void trace_release_pre(struct trace_sb *sb, inode_t *inode, file_t *file, void **context);
void trace_release_post(struct trace_sb *sb, inode_t *inode, file_t *file, int ret, void **context);
void trace_fsync_pre(struct trace_sb *sb, file_t *file, dentry_t *dentry, int datasync, void **context);
void trace_fsync_post(struct trace_sb *sb, file_t *file, dentry_t *dentry, int datasync, int ret, void **context);
void trace_fasync_pre(struct trace_sb *sb, int fd, file_t *file, int flag, void **context);
void trace_fasync_post(struct trace_sb *sb, int fd, file_t *file, int flag, int ret, void **context);
void trace_lock_pre(struct trace_sb *sb, file_t *file, int cmd, struct file_lock *fl, void **context);
void trace_lock_post(struct trace_sb *sb, file_t *file, int cmd, struct file_lock *fl, int ret, void **context);
/* super ops */
void trace_read_inode_pre(struct trace_sb *sb, inode_t *inode, void **context);
void trace_read_inode_post(struct trace_sb *sb, inode_t *inode, void **context);
void trace_write_inode_pre(struct trace_sb *sb, inode_t *inode, int sync, void **context);
void trace_write_inode_post(struct trace_sb *sb, inode_t *inode, int sync, void **context);
void trace_put_inode_pre(struct trace_sb *sb, inode_t *inode, void **context);
void trace_put_inode_post(struct trace_sb *sb, inode_t *inode, void **context);
void trace_delete_inode_pre(struct trace_sb *sb, inode_t *inode, void **context);
void trace_delete_inode_post(struct trace_sb *sb, inode_t *inode, void **context);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
void trace_statfs_pre(struct trace_sb *sb, struct statfs *buf, void **context);
void trace_statfs_post(struct trace_sb *sb, struct statfs *buf, int ret, void **context);
#else
void trace_statfs_pre(struct trace_sb *sb, struct kstatfs *buf, void **context);
void trace_statfs_post(struct trace_sb *sb, struct kstatfs *buf, int ret, void **context);
#endif
void trace_clear_inode_pre(struct trace_sb *sb, inode_t *inode, void **context);
void trace_clear_inode_post(struct trace_sb *sb, inode_t *inode, void **context);
/* dentry ops */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
void trace_d_revalidate_pre(struct trace_sb *sb, dentry_t *dentry, int flags, void **context);
void trace_d_revalidate_post(struct trace_sb *sb, dentry_t *dentry, int flags, int ret, void **context);
#else
void trace_d_revalidate_pre(struct trace_sb *sb, dentry_t *dentry, struct nameidata* nd, void **context);
void trace_d_revalidate_post(struct trace_sb *sb, dentry_t *dentry, struct nameidata* nd, int ret, void **context);
#endif
void trace_d_hash_pre(struct trace_sb *sb, dentry_t *dentry, qstr_t *name, void **context);
void trace_d_hash_post(struct trace_sb *sb, dentry_t *dentry, qstr_t *name, int ret, void **context);
void trace_d_compare_pre(struct trace_sb *sb, dentry_t *dentry, qstr_t *a, qstr_t *b, void **context);
void trace_d_compare_post(struct trace_sb *sb, dentry_t *dentry, qstr_t *a, qstr_t *b, int ret, void **context);
void trace_d_delete_pre(struct trace_sb *sb, dentry_t *dentry, void **context);
void trace_d_delete_post(struct trace_sb *sb, dentry_t *dentry, int ret, void **context);
void trace_d_release_pre(struct trace_sb *sb, dentry_t *dentry, void **context);
void trace_d_release_post(struct trace_sb *sb, dentry_t *dentry, void **context);
void trace_d_iput_pre(struct trace_sb *sb, dentry_t *dentry, inode_t *inode, void **context);
void trace_d_iput_post(struct trace_sb *sb, dentry_t *dentry, inode_t *inode, void **context);

void trace_readpage_pre(struct trace_sb *sb, struct file *file, struct page *page, void **context);
void trace_readpage_post(struct trace_sb *sb, struct file *file, struct page *page, int ret, void **context);
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
void trace_readpages_pre(struct trace_sb *sb, struct file *file, struct address_space* mapping, struct list_head* pages,void **context);
void trace_readpages_post(struct trace_sb *sb, struct file *file, struct address_space* mapping, struct list_head* pages,int ret,void **context);
#endif //#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
void trace_writepage_pre(struct trace_sb *sb, struct page *page, void **context);
void trace_writepage_post(struct trace_sb *sb, struct page *page, int ret, void **context);

void trace_prepare_write_pre(struct trace_sb *sb, struct file *file, struct page *page, unsigned to, unsigned from, void **context);
void trace_prepare_write_post(struct trace_sb *sb, struct file *file, struct page *page, unsigned to, unsigned from, int ret, void **context);
void trace_commit_write_pre(struct trace_sb *sb, struct file *file, struct page *page, unsigned to, unsigned from, void **context);
void trace_commit_write_post(struct trace_sb *sb, struct file *file, struct page *page, unsigned to, unsigned from, int ret, void **context);
void trace_sync_page_pre(struct trace_sb *sb,page_t* page,void** context);
void trace_sync_page_post(struct trace_sb *sb,page_t* page,int ret,void** context);
/* "meta" messages */
void trace_start(struct trace_sb *sb, int id);
void trace_stop(struct trace_sb *sb, int id);

/* customization of assembly driver */
void *add_custom_data(void);
void release_custom_data(void *);

#endif /* __TRACER_H */
