/*
 * Copyright (c) 2003-2007 Erez Zadok
 * Copyright (c) 2003-2007 Stony Brook University
 * Copyright (c) 2003-2007 The Research Foundation of State University of New York
 * Copyright (c) 2004-2007 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2007 Gopala Surya
 */

#include <string.h>
#include "namehash.h"
#include <stdlib.h>

struct namehash *create_namehash(int buckets)
{
	int reqd = sizeof(struct namehash) + buckets * sizeof(struct namehash_entry *);
	struct namehash *tmp = malloc(reqd);
	if (tmp) {
		memset(tmp, 0, reqd);
		tmp->size = buckets;
	}
	return tmp;
}

static void destroy_namehash_entry(struct namehash_entry *entry)
{
	struct names *tmp = entry->namelist;
	while (tmp) {
		entry->namelist = tmp->next;
		free(tmp->name);
		free(tmp);
		tmp = entry->namelist;
	}
}



void destroy_namehash(struct namehash *hash)
{
	int i;
	for (i = 0; i < hash->size; i++)
	{
		struct namehash_entry *tmp = hash->buckets[i];
		struct namehash_entry *tmp2;
		while (tmp) {
			tmp2 = tmp->next;
			destroy_namehash_entry(tmp);
			tmp = tmp2;
		}
	}
}

static int hash(struct namehash *hash, int inum)
{
  if(inum >= 0)
    return (inum % hash->size);
  else
    return ((inum *-1) % hash->size);
}

struct namehash_entry *lookup(struct namehash *hasht, int inum)
{
	int bucket = hash(hasht, inum);
	struct namehash_entry *tmp = hasht->buckets[bucket];
	while(tmp) {
		if (tmp->inum == inum)
			break;
		tmp = tmp->next;
	}
	return tmp;
}

static inline int name_matches(char *full, char *partial)
{
    char *last = strrchr(full, '/');
    if (!last)
	last = full;
    return (strcmp(last, partial) == 0);

}
static void add_name_to_entry(struct namehash_entry *parent, struct namehash_entry *child, char *name)
{
	struct names *tmp;
	struct names *tmp2 = child->namelist;
	while (tmp2) {
	    /* name is already present */
	    if ((!parent || tmp2->parent_inum == parent->inum) && name_matches(tmp2->name, name))
		return;
	    tmp2 = tmp2->next;
	}
	tmp = malloc(sizeof(struct names));

	if (tmp) {
	    tmp->next = child->namelist;
	    child->namelist = tmp;
	    if (parent) {
		tmp->name = (char *)malloc(strlen(name) + strlen(parent->namelist->name) + 2);
		tmp->parent_inum = parent->inum;
		strcpy(tmp->name, parent->namelist->name);
		strcat(tmp->name, "/");
		strcat(tmp->name, name);
	    } else {
		tmp->name = (char *)malloc(strlen(name) + 1);
		tmp->parent_inum = 0;
		strcpy(tmp->name, name);
	    }
	}
}

static struct namehash_entry *create_namehash_entry(struct namehash_entry *parent, int inum, char *name)
{
	struct namehash_entry *tmp = (struct namehash_entry *)malloc(sizeof(struct namehash_entry));
	if (tmp) {
		tmp->inum = inum;
		tmp->namelist = NULL;
		add_name_to_entry(parent, tmp, name);
	}
	return tmp;
}

struct namehash_entry *add_entry(struct namehash *hasht, int inum, int parent_inum, char *name)
{

	int bucket = hash(hasht, inum);
	struct namehash_entry *parent = lookup(hasht, parent_inum);
	struct namehash_entry *current = lookup(hasht, inum);
	if (current) {
		add_name_to_entry(parent, current, name);
	} else {
		current = create_namehash_entry(parent, inum, name);
		if (current) {
			current->next = hasht->buckets[bucket];
			hasht->buckets[bucket] = current;
		}
	}
	return current;
}
